
#include "app.h"

static const char *g_scenePath;

G3D_START_AT_MAIN();

int main(int argc, const char *argv[])
{
    GApp::Settings s;
    s.window.caption = "CS224 Project 0: Ray";

    if (argc > 1)
    {
        g_scenePath = argv[1];
    }
    else
    {
        g_scenePath = "/course/cs224/lib/g3d9/G3D/data/scene/CornellBox-Sphere.scn.any";
        printf("No scene specified, using default: %s\n", g_scenePath);
    }

    return App(s).run();
}


App::App(const GApp::Settings &settings)
    : GApp(settings)
{ }

App::~App() { }

Radiance3 App::raytrace(const Ray &ray, int depth)
{
    Radiance3 final = Radiance3::zero();

    final = Radiance3(0.f, 0.25f, 0.5f); // TODO trace the ray

    return final;
}

void App::threadCallback(int x, int y)
{
    Ray ray = m_world.camera()->worldRay(x + .5f, y + .5f, m_canvas->rect2DBounds());
    m_canvas->set(x, y, raytrace(ray, MAX_DEPTH));
}

static void dispatcher(void *arg)
{
    App *self = (App*)arg;

    int w = self->window()->width(),
        h = self->window()->height();

    printf("Rendering ...");
    GThread::runConcurrently2D(Point2int32(0, 0),
                               Point2int32(w, h),
                               self,
                               &App::threadCallback);
    printf("done\n");
}

void App::onInit()
{
    GApp::showRenderingStats = false;

    m_world.load(g_scenePath);

    m_canvas = Image3::createEmpty(window()->width(),
                                   window()->height());

    m_dispatch = GThread::create("dispatcher", dispatcher, this);
    m_dispatch->start();
}

void App::onCleanup()
{
    m_world.unload();
}

void App::onGraphics(RenderDevice *dev, 
                     Array<shared_ptr<Surface> >& posed3D,
                     Array<shared_ptr<Surface2D> >& posed2D)
{
    Texture::Ref tex = Texture::fromImage("Source", m_canvas);

    FilmSettings s;
    s.setAntialiasingEnabled(false);
    s.setBloomStrength(0);
    s.setGamma(2.060);
    s.setVignetteTopStrength(0);
    s.setVignetteBottomStrength(0);
    m_film->exposeAndRender(renderDevice, s, tex);
}

