#ifndef APP_H
#define APP_H

#include <G3D/G3DAll.h>

#include "world.h"

#define MAX_DEPTH 4

/** The entry point and main window manager */
class App : public GApp
{
public:
    App(const GApp::Settings &settings = GApp::Settings());
    virtual ~App();

    /** TODO Your recursive raytracing function. This is the only function you
      * will need to modify for this assignment.
      *
      * Given the ray to trace and the current recursion depth,
      * trace the ray into m_world and return the radiance along that ray.
      *
      * World::intersect() and World::lineOfSight() will be useful,
      * as will a handful of G3D helper object types (Surfel, Light, Ray,
      * Camera, CoordinateFrame (a.k.a. CFrame), Vector3, Vector4).
      *
      * More info and links to G3D documentation are provided in the handout
      * for this assignment. Read the handout!
      */
    Radiance3 raytrace(const Ray &ray, int depth);

    /** Called once per pixel for raytracing */
    void threadCallback(int x, int y);

    /** Called once at application startup */
    virtual void onInit();

    /** Called once at application shutdown */
    virtual void onCleanup();

    /** Called once per frame to render the scene */
    virtual void onGraphics(RenderDevice *dev, 
                            Array<shared_ptr<Surface> >& posed3D,
                            Array<shared_ptr<Surface2D> >& posed2D);

private:
    World           m_world;    // The scene being rendered
    Image3::Ref     m_canvas;   // Output buffer for raytrace()
    Random          m_random;   // Random number generator
    GThreadRef      m_dispatch; // Spawns rendering threads
};

#endif
